package com.ehsure.ehsurescanner;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.View;
import android.view.Window;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;

import com.google.zxing.BarcodeFormat;
import com.google.zxing.ResultPoint;
import com.journeyapps.barcodescanner.BarcodeCallback;
import com.journeyapps.barcodescanner.BarcodeResult;
import com.journeyapps.barcodescanner.DecoratedBarcodeView;
import com.journeyapps.barcodescanner.DefaultDecoderFactory;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import io.flutter.embedding.android.FlutterActivity;
import pub.devrel.easypermissions.AfterPermissionGranted;
import pub.devrel.easypermissions.EasyPermissions;


public class ScannerActivity extends FlutterActivity implements DecoratedBarcodeView.TorchListener {

    private DecoratedBarcodeView decoratedBarcodeView;
    private ImageView ivFlashlight;
    private boolean isFlashOn = false;

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_er_scanner);
        decoratedBarcodeView = findViewById(R.id.zxing_barcode_scanner);
        ivFlashlight = findViewById(R.id.iv_flashlight);
        TextView tv_input = findViewById(R.id.tv_input);
        tv_input.setOnClickListener(v -> showInputAlertDialog(this::setResultAndFinish));
        decoratedBarcodeView.setTorchListener(this);
        // if the device does not have flashlight in its camera,
        // then remove the switch flashlight button...
        if (!hasFlash()) {
            ivFlashlight.setVisibility(View.GONE);
        }
        ivFlashlight.setOnClickListener(v -> {
            if (v.getId() == R.id.iv_flashlight) {
                if (isFlashOn) {
                    decoratedBarcodeView.setTorchOff();
                } else {
                    decoratedBarcodeView.setTorchOn();
                }
            }
        });
        Collection<BarcodeFormat> formats = Arrays.asList(BarcodeFormat.QR_CODE, BarcodeFormat.CODE_39);
        decoratedBarcodeView.getBarcodeView().setDecoderFactory(new DefaultDecoderFactory(formats));
        decoratedBarcodeView.initializeFromIntent(getIntent());
        decoratedBarcodeView.decodeContinuous(callback);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            requirePermission();
        }
    }

    @AfterPermissionGranted(1)
    private void requirePermission() {
        String[] permissions = {Manifest.permission.CAMERA,};
        if (!EasyPermissions.hasPermissions(this, permissions)) {
            EasyPermissions.requestPermissions(this, "很遗憾你把权限禁用了,部分功能将无法使用!",
                    1, permissions);
        }
    }

    private BarcodeCallback callback = new BarcodeCallback() {
        @Override
        public void barcodeResult(BarcodeResult barcodeResult) {
            String result = barcodeResult.getResult().getText();
            result = getCodeFromUrl(result);
//            beepManager.playBeepSoundAndVibrate();
            setResultAndFinish(result);
        }

        @Override
        public void possibleResultPoints(List<ResultPoint> resultPoints) {
        }
    };

    private void setResultAndFinish(String result){
        Intent intent = new Intent();
        intent.putExtra("result", result);
        setResult(RESULT_OK, intent);
        finish();
    }

    private String getCodeFromUrl(String result) {
        if (TextUtils.isEmpty(result)) {
            return result;
        }
        int index = result.lastIndexOf("=");
        if (index >= 0 && index + 1 != result.length()) {
            result = result.substring(index + 1);
        }
//        if (!TextUtils.isEmpty(result) && result.length() > 25) {
//            result = result.substring(0, 25);
//        }
        return result;
    }

    @Override
    protected void onResume() {
        super.onResume();
        decoratedBarcodeView.resume();
    }

    @Override
    protected void onPause() {
        super.onPause();
        decoratedBarcodeView.pause();
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        return decoratedBarcodeView.onKeyDown(keyCode, event) || super.onKeyDown(keyCode, event);
    }

    /**
     * Check if the device's camera has a Flashlight.
     *
     * @return true if there is Flashlight, otherwise false.
     */
    private boolean hasFlash() {
        return getApplicationContext().getPackageManager()
                .hasSystemFeature(PackageManager.FEATURE_CAMERA_FLASH);
    }

    @Override
    public void onTorchOn() {
        ivFlashlight.setImageResource(R.mipmap.ic_flashlight_on);
        isFlashOn = true;
    }

    @Override
    public void onTorchOff() {
        ivFlashlight.setImageResource(R.mipmap.ic_ope_flashlight);
        isFlashOn = false;
    }

    private void showInputAlertDialog(final OnConfirmListener listener) {
        AlertDialog.Builder builder;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            builder = new AlertDialog.Builder(this,R.style.Theme_AppCompat_Light_Dialog_Alert);
        }else {
            builder = new AlertDialog.Builder(this);
        }
        builder.setTitle("请输入条码值");
        final EditText editText = new EditText(this);
//        editText.setFilters(new InputFilter[] { new InputFilter.LengthFilter(inputLength) }); // 最大输入长度
//        editText.setInputType(InputType.TYPE_CLASS_NUMBER);
        editText.setSingleLine(true);
        editText.setGravity(Gravity.CENTER);
        editText.setTextColor(Color.BLACK);
        Timer timer = new Timer();
        timer.schedule(new TimerTask() {
            @Override
            public void run() {
                InputMethodManager imm = (InputMethodManager) getSystemService(Activity.INPUT_METHOD_SERVICE);
                imm.toggleSoftInput(0, InputMethodManager.HIDE_NOT_ALWAYS);
            }
        }, 500);
        builder.setView(editText);
        builder.setPositiveButton("确定", null);
        builder.setNegativeButton("取消", (dialog, which) -> {
            dialog.dismiss();
            hideSoftInput(editText);
        });
        final AlertDialog dialog = builder.create();
        dialog.show();
        dialog.getButton(AlertDialog.BUTTON_POSITIVE).setOnClickListener(v -> {
            String input = editText.getText().toString();
            if (!TextUtils.isEmpty(input)) {
                listener.confirmCallback(input);
                dialog.dismiss();
                hideSoftInput(editText);
            } else
                Toast.makeText(this,"请输入条码值",Toast.LENGTH_SHORT).show();
        });
    }

    private interface OnConfirmListener {
        void confirmCallback(String text);
    }

    private void hideSoftInput(EditText edit) {
        edit.clearFocus();
        InputMethodManager inputmanger = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
        inputmanger.hideSoftInputFromWindow(edit.getWindowToken(), 0);
    }

}
